Package: readr
Title: Read Rectangular Text Data
Version: 2.2.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Jim", "Hester", role = "aut"),
    person("Romain", "Francois", role = "ctb"),
    person("Jennifer", "Bryan", , "jenny@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6983-2759")),
    person("Shelby", "Bearrows", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49")),
    person("https://github.com/mandreyel/", role = "cph",
           comment = "mio library"),
    person("Jukka", "Jylänki", role = c("ctb", "cph"),
           comment = "grisu3 implementation"),
    person("Mikkel", "Jørgensen", role = c("ctb", "cph"),
           comment = "grisu3 implementation")
  )
Description: The goal of 'readr' is to provide a fast and friendly way to
    read rectangular data (like 'csv', 'tsv', and 'fwf').  It is designed
    to flexibly parse many types of data found in the wild, while still
    cleanly failing when data unexpectedly changes.
License: MIT + file LICENSE
URL: https://readr.tidyverse.org, https://github.com/tidyverse/readr
BugReports: https://github.com/tidyverse/readr/issues
Depends: R (>= 4.1)
Imports: cli, clipr, crayon, glue, hms (>= 0.4.1), lifecycle, methods,
        R6, rlang, tibble, utils, vroom (>= 1.7.0), withr
Suggests: covr, curl, datasets, knitr, rmarkdown, spelling, stringi,
        testthat (>= 3.2.0), tzdb (>= 0.1.1), waldo, xml2
LinkingTo: cpp11, tzdb (>= 0.1.1)
VignetteBuilder: knitr
Config/Needs/website: tidyverse, tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: false
Config/usethis/last-upkeep: 2025-11-14
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-02-19 07:07:31 UTC; jenny
Author: Hadley Wickham [aut],
  Jim Hester [aut],
  Romain Francois [ctb],
  Jennifer Bryan [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6983-2759>),
  Shelby Bearrows [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>),
  https://github.com/mandreyel/ [cph] (mio library),
  Jukka Jylänki [ctb, cph] (grisu3 implementation),
  Mikkel Jørgensen [ctb, cph] (grisu3 implementation)
Maintainer: Jennifer Bryan <jenny@posit.co>
Repository: CRAN
Date/Publication: 2026-02-19 18:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 09:32:56 UTC; unix
Archs: readr.so.dSYM
