# Generated by roxygen2: do not edit by hand

S3method("[",spec_tbl_df)
S3method(as.character,col_spec)
S3method(as.col_spec,"NULL")
S3method(as.col_spec,character)
S3method(as.col_spec,col_spec)
S3method(as.col_spec,data.frame)
S3method(as.col_spec,default)
S3method(as.col_spec,list)
S3method(as.data.frame,spec_tbl_df)
S3method(as_chunk_callback,'function')
S3method(as_chunk_callback,ChunkCallback)
S3method(as_chunk_callback,R6ClassGenerator)
S3method(as_tibble,spec_tbl_df)
S3method(format,col_spec)
S3method(output_column,POSIXt)
S3method(output_column,default)
S3method(output_column,double)
S3method(print,col_spec)
S3method(print,collector)
S3method(print,date_names)
S3method(print,locale)
S3method(str,col_spec)
S3method(type_to_col,Date)
S3method(type_to_col,POSIXct)
S3method(type_to_col,default)
S3method(type_to_col,double)
S3method(type_to_col,factor)
S3method(type_to_col,hms)
S3method(type_to_col,integer)
S3method(type_to_col,logical)
export(AccumulateCallback)
export(ChunkCallback)
export(DataFrameCallback)
export(ListCallback)
export(SideEffectChunkCallback)
export(as.col_spec)
export(clipboard)
export(col_character)
export(col_date)
export(col_datetime)
export(col_double)
export(col_factor)
export(col_guess)
export(col_integer)
export(col_logical)
export(col_number)
export(col_skip)
export(col_time)
export(cols)
export(cols_condense)
export(cols_only)
export(count_fields)
export(datasource)
export(date_names)
export(date_names_lang)
export(date_names_langs)
export(default_locale)
export(edition_get)
export(format_csv)
export(format_csv2)
export(format_delim)
export(format_tsv)
export(fwf_cols)
export(fwf_empty)
export(fwf_positions)
export(fwf_widths)
export(guess_encoding)
export(guess_parser)
export(local_edition)
export(locale)
export(output_column)
export(parse_character)
export(parse_date)
export(parse_datetime)
export(parse_double)
export(parse_factor)
export(parse_guess)
export(parse_integer)
export(parse_logical)
export(parse_number)
export(parse_time)
export(parse_vector)
export(problems)
export(read_builtin)
export(read_csv)
export(read_csv2)
export(read_csv2_chunked)
export(read_csv_chunked)
export(read_delim)
export(read_delim_chunked)
export(read_file)
export(read_file_raw)
export(read_fwf)
export(read_lines)
export(read_lines_chunked)
export(read_lines_raw)
export(read_lines_raw_chunked)
export(read_log)
export(read_rds)
export(read_table)
export(read_tsv)
export(read_tsv_chunked)
export(readr_example)
export(readr_threads)
export(should_read_lazy)
export(should_show_types)
export(show_progress)
export(spec)
export(spec_csv)
export(spec_csv2)
export(spec_delim)
export(spec_table)
export(spec_tsv)
export(stop_for_problems)
export(tokenize)
export(tokenizer_csv)
export(tokenizer_delim)
export(tokenizer_fwf)
export(tokenizer_line)
export(tokenizer_log)
export(tokenizer_tsv)
export(tokenizer_ws)
export(type_convert)
export(with_edition)
export(write_csv)
export(write_csv2)
export(write_delim)
export(write_excel_csv)
export(write_excel_csv2)
export(write_file)
export(write_lines)
export(write_rds)
export(write_tsv)
import(rlang)
importFrom(R6,R6Class)
importFrom(glue,glue)
importFrom(hms,hms)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(methods,setOldClass)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
useDynLib(readr, .registration = TRUE)
