# grouping and decimal marks must be different

    Code
      locale(grouping_mark = ".", decimal_mark = ".")
    Condition
      Error:
      ! `decimal_mark` and `grouping_mark` must be different

