# standardise_path works

    Code
      standardise_path("https://foo/bar.bz2")
    Condition
      Error:
      ! Reading from remote `bz2` compressed files is not supported,
        download the files locally first.

